ig.module(
    'game.entities.fossil'
)
.requires(
    'game.entities.base'
)
.defines(function () {
    EntityFossil = EntityBase.extend({
        collides: ig.Entity.COLLIDES.NEVER,
        
        type: ig.Entity.TYPE.B,
        
        zIndex: 20,
        
        frameOffset: 0, // Frame offset for defining animations
        
        what: 'fossil',
        
        anim: 'orange1',
        animSet: false,
        
        color: 'orange',
        
        bad: false,
        perfect: true,
        
        init: function (x, y, settings) {
            this.parent(x, y, settings);
            
            if (ig.game.setCellEntityXY) {
                ig.game.setCellEntityXY(this, this.pos.x, this.pos.y);
            }
            
            this.determineAnim();
        },
        
    	createAnimations: function () {
    	    //this.addAnim('yellow', 1, [80]);
    	    this.addAnim('orange1', 1, [120]);
    	    this.addAnim('orange2', 1, [121]);
    	    this.addAnim('red1', 1, [160]);
    	    this.addAnim('red2', 1, [161]);
    	    this.addAnim('red3', 1, [162]);
    	    this.addAnim('red4', 1, [163]);
    	    this.addAnim('red5', 1, [164]);
    	    this.addAnim('red6', 1, [165]);
    	    this.addAnim('red7', 1, [166]);
    	    this.addAnim('red8', 1, [167]);
    	    this.addAnim('red9', 1, [168]);
    	    this.addAnim('purple1', 1, [200]);
    	    this.addAnim('purple2', 1, [201]);
    	    this.addAnim('purple3', 1, [202]);
    	},
        
        determineAnim: function () {
    	    if (!this.animSet && this.anim) {
    	        this.currentAnim = this.anims[this.anim];
    	        this.animSet = true;
    	        
    	        this.color = this.anim.replace(/[0-9]+/, '');
    	    }
    	},
        
        // TODO: this is an obvious indication that it would be better to subclass fossils by color
        isPerfect: function () {            
            if (this.color == 'orange') {
                return true;
            }
            
            var i, surrounding = ig.game.getSurroundingCellsXY(this.pos.x, this.pos.y);
            
            i = 8;
            while (i--) {
                if (!surrounding[i]) {
                    surrounding[i] = {anim: ''};
                }
            }
            
            var UP = 1, LEFT = 3, RIGHT = 4, DOWN = 6;
            
            if (this.color == 'red') {
                switch (this.anim) {
                    case 'red1':
                        return (surrounding[RIGHT].anim == 'red2' && surrounding[DOWN].anim == 'red4');
                        
                    case 'red2':
                        return (surrounding[LEFT].anim == 'red1' && surrounding[DOWN].anim == 'red5' && surrounding[RIGHT].anim == 'red3');
                        
                    case 'red3':
                        return (surrounding[LEFT].anim == 'red2' && surrounding[DOWN].anim == 'red6');
                        
                    case 'red4':
                        return (surrounding[RIGHT].anim == 'red5' && surrounding[UP].anim == 'red1');

                    case 'red5':
                        return (surrounding[LEFT].anim == 'red4' && surrounding[UP].anim == 'red2' && surrounding[RIGHT].anim == 'red6');

                    case 'red6':
                        return (surrounding[LEFT].anim == 'red5' && surrounding[UP].anim == 'red3');
                    
                    
                    case 'red7':
                        return (surrounding[DOWN].anim == 'red8');

                    case 'red8':
                        return (surrounding[UP].anim == 'red7' && surrounding[DOWN].anim == 'red9');

                    case 'red9':
                        return (surrounding[UP].anim == 'red8');
                }
                
            } else if (this.color == 'purple') {
                switch (this.anim) {
                    case 'purple1':
                        return (surrounding[RIGHT].anim == 'purple2');
                        
                    case 'purple2':
                        return (surrounding[LEFT].anim == 'purple1' && surrounding[RIGHT].anim == 'purple3');
                        
                    case 'purple3':
                        return (surrounding[LEFT].anim == 'purple2');
                }
            }
            
            return false;
        },
        
        update: function () {
            var myCell = ig.game.getCell(this.pos.x, this.pos.y);
            
            if (
                /*
                (this.anim == 'yellow' && (myCell.row != 1))
                || (this.anim == 'orange' && (myCell.row < 2 || myCell.row > 3))
                || (this.anim == 'red' && (myCell.row < 4 || myCell.row > 6))
                || (this.anim == 'purple' && (myCell.row < 7))
                */
                (this.color == 'orange' && (myCell.row > 2))
                || (this.color == 'red' && (myCell.row < 3 || myCell.row > 6))
                || (this.color == 'purple' && (myCell.row < 7))
            ) {
                if (!this.bad) {
                    this.bad = true;
                    ig.badFossils++;
                }
            } else {
                if (this.bad) {
                    this.bad = false;
                    ig.badFossils--;
                }
            }
            
            if (this.isPerfect()) {
                if (!this.perfect) {
                    this.perfect = true;
                    ig.imperfectFossils--;
                }
            } else {
                if (this.perfect) {
                    this.perfect = false;
                    ig.imperfectFossils++;
                }
            }
            
            if (ig.badFossils == 0) {
                ig.stage = 'levelwin';
                
                if (ig.imperfectFossils == 0) {
                    ig.stage = 'levelperfect';
                }
                
                if (ig.game.currentLevel + 1 >= ig.game.levels.length) {
                    ig.stage = 'gamewin';
                }
            }
            
            this.parent();
        }
    });
});